Package: HRW
Version: 1.0-6
Date: 2026-01-23
Title: Datasets, Functions and Scripts for Semiparametric Regression
        Supporting Harezlak, Ruppert & Wand (2018)
Authors@R: c(person("Jaroslaw", "Harezlak", role = "aut", email = "harezlak@iu.edu"), 
              person("David", "Ruppert", role = "aut", email = "dr24@cornell.edu"), 
              person("Matt P.", "Wand", role = c("aut","cre"), email = "matt.wand@uts.edu.au"))
Maintainer: Matt P. Wand <matt.wand@uts.edu.au>
Imports: KernSmooth, grDevices, graphics, splines, stats
Suggests: fields, lattice, mgcv, Ecdat, LowRankQP, VGAM, e1071, fda,
        fda.usc, mlbench, nlme, quantreg, refund, rgl, rstan
Description: The book "Semiparametric Regression with R" by J. Harezlak, D. Ruppert & M.P. Wand (2018, Springer; ISBN: 978-1-4939-8851-8) makes use of datasets and scripts to explain semiparametric regression concepts. Each of the book's scripts are contained in this package as well as datasets that are not within other R packages. Functions that aid semiparametric regression analysis are also included.
License: GPL (>= 2)
LazyData: TRUE
LazyDataCompression: xz
NeedsCompilation: no
Depends: R (>= 4.5.0)
Packaged: 2026-01-23 04:09:49 UTC; mwand
Author: Jaroslaw Harezlak [aut],
  David Ruppert [aut],
  Matt P. Wand [aut, cre]
Repository: CRAN
Date/Publication: 2026-01-23 06:41:29 UTC
Built: R 4.6.0; ; 2026-04-26 17:06:33 UTC; unix
